@@ -14,7 +14,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_dcache.c,v 1.42.2.9 2005/03/20 20:09:13 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_dcache.c,v 1.42.2.24 2008/08/26 14:02:10 shadow Exp $");
 
 #include "afs/sysincludes.h"	/*Standard vendor system headers */
 #include "afsincludes.h"	/*AFS-based standard headers */
@@ -119,9 +119,6 @@ struct afs_cacheOps afs_MemCacheOps = {
 int cacheDiskType;		/*Type of backing disk for cache */
 struct afs_cacheOps *afs_cacheType;
 
-
-
-
 /*
  * afs_StoreWarn
  *
@@ -226,16 +223,14 @@ afs_CacheTruncateDaemon(void)
     u_int counter;
     u_int cb_lowat;
     u_int dc_hiwat =
-	(100 - CM_DCACHECOUNTFREEPCT +
-	 CM_DCACHEEXTRAPCT) * afs_cacheFiles / 100;
+	PERCENT((100 - CM_DCACHECOUNTFREEPCT + CM_DCACHEEXTRAPCT), afs_cacheFiles);
     afs_min_cache =
 	(((10 * AFS_CHUNKSIZE(0)) + afs_fsfragsize) & ~afs_fsfragsize) >> 10;
 
     osi_GetuTime(&CTD_stats.CTD_afterSleep);
     afs_TruncateDaemonRunning = 1;
     while (1) {
-	cb_lowat = ((CM_DCACHESPACEFREEPCT - CM_DCACHEEXTRAPCT)
-		    * afs_cacheBlocks) / 100;
+	cb_lowat = PERCENT((CM_DCACHESPACEFREEPCT - CM_DCACHEEXTRAPCT), afs_cacheBlocks);
 	MObtainWriteLock(&afs_xdcache, 266);
 	if (afs_CacheTooFull) {
 	    int space_needed, slots_needed;
@@ -392,6 +387,11 @@ afs_GetDownD(int anumber, int *aneedSpace)
     afs_hyper_t maxVictimTime;	/* youngest (largest LRU time) victim */
     afs_uint32 maxVictimPtr;	/* where it is */
     int discard;
+    int vfslocked;
+
+#if defined(AFS_FBSD80_ENV) && !defined(UKERNEL)
+    vfslocked = VFS_LOCK_GIANT(afs_globalVFS);
+#endif
 
     AFS_STATCNT(afs_GetDownD);
     if (CheckLock(&afs_xdcache) != -1)
@@ -401,9 +401,14 @@ afs_GetDownD(int anumber, int *aneedSpace)
      * because we should try to free space even if anumber <=0 */
     if (!aneedSpace || *aneedSpace <= 0) {
 	anumber -= afs_freeDCCount;
-	if (anumber <= 0)
+	if (anumber <= 0) {
+#if defined(AFS_FBSD80_ENV) && !defined(UKERNEL)
+	  VFS_UNLOCK_GIANT(vfslocked);
+#endif
 	    return;		/* enough already free */
+	}
     }
+
     /* bounds check parameter */
     if (anumber > MAXATONCE)
 	anumber = MAXATONCE;	/* all we can do */
@@ -586,7 +591,7 @@ afs_GetDownD(int anumber, int *aneedSpace)
 			MReleaseWriteLock(&afs_xdcache);
 		    }
 
-		    AFS_FAST_RELE(tvc);
+		    afs_PutVCache(tvc); /*XXX was AFS_FAST_RELE?*/
 		    MObtainWriteLock(&afs_xdcache, 528);
 		    if (afs_indexFlags[tdc->index] &
 			(IFDataMod | IFDirtyPages | IFAnyPages))
@@ -625,9 +630,7 @@ afs_GetDownD(int anumber, int *aneedSpace)
 			       ICL_TYPE_POINTER, tvc, ICL_TYPE_INT32, 3,
 			       ICL_TYPE_INT32, tdc->index, ICL_TYPE_OFFSET,
 			       ICL_HANDLE_OFFSET(tchunkoffset));
-#ifndef	AFS_DEC_ENV
 		    AFS_STATCNT(afs_gget);
-#endif
 		    afs_HashOutDCache(tdc);
 		    if (tdc->f.chunkBytes != 0) {
 			discard = 1;
@@ -664,6 +667,11 @@ afs_GetDownD(int anumber, int *aneedSpace)
 		break;
 	}
     }				/* big while loop */
+
+#if defined(AFS_FBSD80_ENV) && !defined(UKERNEL)
+    VFS_UNLOCK_GIANT(vfslocked);
+#endif
+
     return;
 
 }				/*afs_GetDownD */
@@ -682,9 +690,7 @@ afs_HashOutDCache(struct dcache *adc)
 {
     int i, us;
 
-#ifndef	AFS_DEC_ENV
     AFS_STATCNT(afs_glink);
-#endif
     /* we know this guy's in the LRUQ.  We'll move dude into DCQ below */
     DZap(adc);
     /* if this guy is in the hash table, pull him out */
@@ -784,7 +790,7 @@ afs_FlushDCache(register struct dcache *adc)
 
     if (afs_WaitForCacheDrain) {
 	if (afs_blocksUsed <=
-	    (CM_CACHESIZEDRAINEDPCT * afs_cacheBlocks) / 100) {
+	    PERCENT(CM_CACHESIZEDRAINEDPCT, afs_cacheBlocks)) {
 	    afs_WaitForCacheDrain = 0;
 	    afs_osi_Wakeup(&afs_WaitForCacheDrain);
 	}
@@ -817,7 +823,7 @@ afs_FreeDCache(register struct dcache *adc)
 
     if (afs_WaitForCacheDrain) {
 	if ((afs_blocksUsed - afs_blocksDiscarded) <=
-	    (CM_CACHESIZEDRAINEDPCT * afs_cacheBlocks) / 100) {
+	    PERCENT(CM_CACHESIZEDRAINEDPCT, afs_cacheBlocks)) {
 	    afs_WaitForCacheDrain = 0;
 	    afs_osi_Wakeup(&afs_WaitForCacheDrain);
 	}
@@ -867,7 +873,7 @@ afs_DiscardDCache(register struct dcache *adc)
 
     if (afs_WaitForCacheDrain) {
 	if ((afs_blocksUsed - afs_blocksDiscarded) <=
-	    (CM_CACHESIZEDRAINEDPCT * afs_cacheBlocks) / 100) {
+	    PERCENT(CM_CACHESIZEDRAINEDPCT, afs_cacheBlocks)) {
 	    afs_WaitForCacheDrain = 0;
 	    afs_osi_Wakeup(&afs_WaitForCacheDrain);
 	}
@@ -950,7 +956,7 @@ afs_MaybeFreeDiscardedDCache(void)
 
     while (afs_blocksDiscarded
 	   && (afs_blocksUsed >
-	       (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100)) {
+	       PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks))) {
 	afs_FreeDiscardedDCache();
     }
     return 0;
@@ -1024,17 +1030,6 @@ afs_GetDownDSlot(int anumber)
 #endif
 	    }
 
-	    tdc->stamp = 0;
-#ifdef IHINT
-	    if (tdc->ihint) {
-		struct osi_file *f = (struct osi_file *)tdc->ihint;
-		tdc->ihint = 0;
-		afs_UFSClose(f);
-		nihints--;
-	    }
-#endif /* IHINT */
-
-
 	    /* finally put the entry in the free list */
 	    afs_indexTable[ix] = NULL;
 	    afs_indexFlags[ix] &= ~IFEverUsed;
@@ -1221,14 +1216,14 @@ afs_FindDCache(register struct vcache *avc, afs_size_t abyte)
 	}
 	index = afs_dcnextTbl[index];
     }
-    MReleaseWriteLock(&afs_xdcache);
     if (index != NULLIDX) {
 	hset(afs_indexTimes[tdc->index], afs_indexCounter);
 	hadd32(afs_indexCounter, 1);
+	MReleaseWriteLock(&afs_xdcache);
 	return tdc;
-    } else
-	return NULL;
-
+    } 
+    MReleaseWriteLock(&afs_xdcache);
+    return NULL;
 }				/*afs_FindDCache */
 
 
@@ -1487,14 +1482,8 @@ void
 updateV2DC(int lockVc, struct vcache *v, struct dcache *d, int src)
 {
     if (!lockVc || 0 == NBObtainWriteLock(&v->lock, src)) {
-	if (hsame(v->m.DataVersion, d->f.versionNo) && v->callback) {
-	    v->quick.dc = d;
-	    v->quick.stamp = d->stamp = MakeStamp();
-	    v->quick.minLoc = AFS_CHUNKTOBASE(d->f.chunk);
-	    /* Don't think I need these next two lines forever */
-	    v->quick.len = d->f.chunkBytes;
-	    v->h1.dchint = d;
-	}
+	if (hsame(v->m.DataVersion, d->f.versionNo) && v->callback)
+	    v->dchint = d;
 	if (lockVc)
 	    ReleaseWriteLock(&v->lock);
     }
@@ -1586,7 +1575,7 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
     shortcut = 0;
 
     /* check hints first! (might could use bcmp or some such...) */
-    if ((tdc = avc->h1.dchint)) {
+    if ((tdc = avc->dchint)) {
 	int dcLocked;
 
 	/*
@@ -1839,7 +1828,8 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	       ICL_HANDLE_OFFSET(Position));
     if ((aflags & 4) && (hiszero(avc->m.DataVersion)))
 	doAdjustSize = 1;
-    if ((aflags & 4) && (abyte == Position) && (tlen >= size))
+    if ((AFS_CHUNKTOBASE(chunk) >= avc->m.Length) ||
+	 ((aflags & 4) && (abyte == Position) && (tlen >= size)))
 	overWriteWholeChunk = 1;
     if (doAdjustSize || overWriteWholeChunk) {
 #if	defined(AFS_AIX32_ENV) || defined(AFS_SGI_ENV)
@@ -1857,7 +1847,7 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 #endif /* AFS_SGI_ENV */
 	if (AFS_CHUNKTOBASE(chunk) + adjustsize >= avc->m.Length &&
 #else /* defined(AFS_AIX32_ENV) || defined(AFS_SGI_ENV) */
-#if	defined(AFS_SUN_ENV)  || defined(AFS_OSF_ENV)
+#if	defined(AFS_SUN5_ENV)  || defined(AFS_OSF_ENV)
 	if ((doAdjustSize || (AFS_CHUNKTOBASE(chunk) >= avc->m.Length)) &&
 #else
 	if (AFS_CHUNKTOBASE(chunk) >= avc->m.Length &&
@@ -1990,7 +1980,7 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	/* Sleep here when cache needs to be drained. */
 	if (setLocks && !slowPass
 	    && (afs_blocksUsed >
-		(CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100)) {
+		PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks))) {
 	    /* Make sure truncate daemon is running */
 	    afs_MaybeWakeupTruncateDaemon();
 	    ObtainWriteLock(&tdc->tlock, 614);
@@ -1999,7 +1989,7 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	    ReleaseWriteLock(&tdc->lock);
 	    ReleaseReadLock(&avc->lock);
 	    while ((afs_blocksUsed - afs_blocksDiscarded) >
-		   (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100) {
+		   PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
 		afs_WaitForCacheDrain = 1;
 		afs_osi_Sleep(&afs_WaitForCacheDrain);
 	    }
@@ -2040,20 +2030,7 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	 * fetch the whole file.
 	 */
 	DZap(tdc);	/* pages in cache may be old */
-#ifdef  IHINT
-	if (file = tdc->ihint) {
-	    if (tdc->f.inode == file->inum)
-		usedihint++;
-	    else {
-		tdc->ihint = 0;
-		afs_UFSClose(file);
-		file = 0;
-		nihints--;
-		file = osi_UFSOpen(tdc->f.inode);
-	    }
-	} else
-#endif /* IHINT */
-	    file = afs_CFileOpen(tdc->f.inode);
+	file = afs_CFileOpen(tdc->f.inode);
 	afs_RemoveVCB(&avc->fid);
 	tdc->f.states |= DWriting;
 	tdc->dflags |= DFFetching;
@@ -2420,17 +2397,19 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	    }
 	    ReleaseWriteLock(&tdc->lock);
 	    afs_PutDCache(tdc);
-	    ObtainWriteLock(&afs_xcbhash, 454);
-	    afs_DequeueCallback(avc);
-	    avc->states &= ~(CStatd | CUnique);
-	    ReleaseWriteLock(&afs_xcbhash);
-	    if (avc->fid.Fid.Vnode & 1 || (vType(avc) == VDIR))
-		osi_dnlc_purgedp(avc);
-	    /*
-	     * Locks held:
-	     * avc->lock(W); assert(!setLocks || slowPass)
-	     */
-	    osi_Assert(!setLocks || slowPass);
+	    if (!afs_IsDynroot(avc)) {
+		ObtainWriteLock(&afs_xcbhash, 454);
+		afs_DequeueCallback(avc);
+		avc->states &= ~(CStatd | CUnique);
+		ReleaseWriteLock(&afs_xcbhash);
+		if (avc->fid.Fid.Vnode & 1 || (vType(avc) == VDIR))
+		    osi_dnlc_purgedp(avc);
+		/*
+		 * Locks held:
+		 * avc->lock(W); assert(!setLocks || slowPass)
+		 */
+		osi_Assert(!setLocks || slowPass);
+	    }
 	    tdc = NULL;
 	    goto done;
 	}
@@ -2774,7 +2753,7 @@ afs_UFSGetDSlot(register afs_int32 aslot, register struct dcache *tmpdc)
     if (CheckLock(&afs_xdcache) != -1)
 	osi_Panic("getdslot nolock");
     if (aslot < 0 || aslot >= afs_cacheFiles)
-	osi_Panic("getdslot slot");
+	osi_Panic("getdslot slot %d (of %d)", aslot, afs_cacheFiles);
     tdc = afs_indexTable[aslot];
     if (tdc) {
 	QRemove(&tdc->lruq);	/* move to queue head */
@@ -2813,7 +2792,6 @@ afs_UFSGetDSlot(register afs_int32 aslot, register struct dcache *tmpdc)
     } else {
 	tdc = tmpdc;
 	tdc->f.states = 0;
-	tdc->ihint = 0;
     }
 
     /*
@@ -2897,6 +2875,7 @@ afs_WriteDCache(register struct dcache *adc, int atime)
     if (cacheDiskType == AFS_FCACHE_TYPE_MEM)
 	return 0;
     AFS_STATCNT(afs_WriteDCache);
+    osi_Assert(WriteLocked(&afs_xdcache));
     if (atime)
 	adc->f.modTime = osi_Time();
     /*
@@ -3025,11 +3004,7 @@ afs_InitCacheFile(char *afile, ino_t ainode)
 	dput(filevp);
 #else
 	tdc->f.inode = afs_vnodeToInumber(filevp);
-#ifdef AFS_DEC_ENV
-	grele(filevp);
-#else
 	AFS_RELE(filevp);
-#endif
 #endif /* AFS_LINUX22_ENV */
     } else {
 	tdc->f.inode = ainode;
