@@ -48,6 +48,7 @@
 #define SYS_NAME_ID_hp_ux11i             416
 #define SYS_NAME_ID_ia64_hpux1122	 417
 #define SYS_NAME_ID_ia64_hpux1123	 418
+#define SYS_NAME_ID_hp_ux1123		 419
 
 #define SYS_NAME_ID_mac2_51		 500
 #define SYS_NAME_ID_mac_aux10		 501
@@ -57,6 +58,10 @@
 #define SYS_NAME_ID_ppc_darwin_14        505
 #define SYS_NAME_ID_ppc_darwin_60        506
 #define SYS_NAME_ID_ppc_darwin_70        507
+#define SYS_NAME_ID_ppc_darwin_80        508
+#define SYS_NAME_ID_x86_darwin_80        509
+#define SYS_NAME_ID_ppc_darwin_90        510
+#define SYS_NAME_ID_x86_darwin_90        511
 
 #define SYS_NAME_ID_next_mach20		 601
 #define SYS_NAME_ID_next_mach30		 602
@@ -67,6 +72,8 @@
 #define SYS_NAME_ID_rs_aix51		 704
 #define SYS_NAME_ID_rs_aix43		 705
 #define SYS_NAME_ID_rs_aix52		 706
+#define SYS_NAME_ID_rs_aix53		 707
+#define SYS_NAME_ID_rs_aix61		 708
 
 #define SYS_NAME_ID_sun3_411		 906
 #define SYS_NAME_ID_sun3x_411		 912
@@ -93,6 +100,7 @@
 #define SYS_NAME_ID_sun4x_58		 939
 #define SYS_NAME_ID_sun4x_59		 940
 #define SYS_NAME_ID_sun4x_510		 941
+#define SYS_NAME_ID_sun4x_511		 942
 
 /* Sigh. If I leave a gap here IBM will do this sequentially. If I don't
    they won't allocate sunx86 IDs at all. So leave a gap and pray. */
@@ -100,6 +108,7 @@
 #define SYS_NAME_ID_sunx86_58		 951
 #define SYS_NAME_ID_sunx86_59		 952
 #define SYS_NAME_ID_sunx86_510		 953
+#define SYS_NAME_ID_sunx86_511		 954
 
 #define SYS_NAME_ID_vax_ul4		1003
 #define SYS_NAME_ID_vax_ul40		1004
@@ -148,10 +157,12 @@
 #define SYS_NAME_ID_sparc64_linux2	1800
 #define SYS_NAME_ID_sparc64_linux22	1801
 #define SYS_NAME_ID_sparc64_linux24	1802
+#define SYS_NAME_ID_sparc64_linux26	1803
 
 #define SYS_NAME_ID_s390_linux2         1900
 #define SYS_NAME_ID_s390_linux22        1901
 #define SYS_NAME_ID_s390_linux24        1902
+#define SYS_NAME_ID_s390_linux26        1907
 #define SYS_NAME_ID_s390x_linux2         1903
 #define SYS_NAME_ID_s390x_linux22        1904
 #define SYS_NAME_ID_s390x_linux24        1905
@@ -160,6 +171,7 @@
 #define SYS_NAME_ID_alpha_linux_2       2000
 #define SYS_NAME_ID_alpha_linux_22      2001
 #define SYS_NAME_ID_alpha_linux_24      2002
+#define SYS_NAME_ID_alpha_linux_26      2003
 
 #define SYS_NAME_ID_i386_fbsd_42        2100
 #define SYS_NAME_ID_i386_fbsd_43        2101
@@ -171,6 +183,11 @@
 #define SYS_NAME_ID_i386_fbsd_51        2106
 #define SYS_NAME_ID_i386_fbsd_52        2107
 #define SYS_NAME_ID_i386_fbsd_53        2108
+#define SYS_NAME_ID_i386_fbsd_60        2112
+#define SYS_NAME_ID_i386_fbsd_61        2113
+#define SYS_NAME_ID_i386_fbsd_62        2114
+#define SYS_NAME_ID_i386_fbsd_70        2115
+#define SYS_NAME_ID_i386_fbsd_80        2116
 
 #define SYS_NAME_ID_ia64_linux2		2200
 #define SYS_NAME_ID_ia64_linux22	2201
@@ -201,6 +218,21 @@
 #define SYS_NAME_ID_macppc_nbsd20	2524
 #define SYS_NAME_ID_i386_nbsd21		2525
 #define SYS_NAME_ID_i386_nbsd30		2526
+#define SYS_NAME_ID_amd64_nbsd20	2527
+#define SYS_NAME_ID_i386_nbsd40 2528
+#define SYS_NAME_ID_i386_nbsd50 2529
+#define SYS_NAME_ID_sparc64_nbsd30 2530
+#define SYS_NAME_ID_sparc64_nbsd40 2531
+#define SYS_NAME_ID_sparc64_nbsd50 2532
+#define SYS_NAME_ID_amd64_nbsd30 2533
+#define SYS_NAME_ID_amd64_nbsd40 2534
+#define SYS_NAME_ID_amd64_nbsd50 2535
+#define SYS_NAME_ID_alpha_nbsd30 2536
+#define SYS_NAME_ID_alpha_nbsd40 2537
+#define SYS_NAME_ID_alpha_nbsd50 2538
+#define SYS_NAME_ID_macppc_nbsd30 2539
+#define SYS_NAME_ID_macppc_nbsd40 2540
+#define SYS_NAME_ID_macppc_nbsd50 2541
 
 #define SYS_NAME_ID_i386_obsd31		2600
 #define SYS_NAME_ID_i386_obsd32		2601
@@ -208,6 +240,11 @@
 #define SYS_NAME_ID_i386_obsd34		2603
 #define SYS_NAME_ID_i386_obsd35		2604
 #define SYS_NAME_ID_i386_obsd36		2605
+#define SYS_NAME_ID_i386_obsd37		2606
+#define SYS_NAME_ID_i386_obsd38		2607
+#define SYS_NAME_ID_i386_obsd39		2608
+#define SYS_NAME_ID_i386_obsd40         2609
+#define SYS_NAME_ID_i386_obsd41         2610
 
 #define SYS_NAME_ID_amd64_linux2        2700
 #define SYS_NAME_ID_amd64_linux22       2701
@@ -226,9 +263,14 @@
 
 #define SYS_NAME_ID_amd64_fbsd_53        3008
 
-#define SYS_NAME_ID_amd64_w2k           3100
+#define SYS_NAME_ID_amd64_w2k           3400
 
-#define SYS_NAME_ID_i64_w2k             3200
+#define SYS_NAME_ID_i64_w2k             3500
+
+#define SYS_NAME_ID_arm_linux2          3800
+#define SYS_NAME_ID_arm_linux22         3801
+#define SYS_NAME_ID_arm_linux24         3802
+#define SYS_NAME_ID_arm_linux26         3803
 
 /*
  * Placeholder to keep system-wide standard flags since this file is included by all 
