@@ -10,42 +10,38 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/kauth/kpwvalid.c,v 1.5 2001/10/05 21:17:18 shadow Exp $");
+RCSID
+    ("$Header: /cvs/openafs/src/kauth/kpwvalid.c,v 1.6.2.3 2008/10/27 23:54:09 shadow Exp $");
 
 #include <stdio.h>
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #ifdef AFS_NT40_ENV
 #include <WINNT/afsevent.h>
 #endif
 #include <afs/afsutil.h>
 
 #include "AFS_component_version_number.c"
-  
-  /* returns 0 if the password is long enough, otherwise non-zero  */
-main(int argc, char *argv[] )
-{
-  char oldpassword[512];
-  char password[512];
-  int rc;
 
-  if (fgets(oldpassword, 512, stdin)) 
-    while (fgets(password, 512, stdin)) {
-       if (strlen(password) > 8) { /* password includes a newline */ 
-	 rc = 0;
-	 fputs("0\n",stdout);
-	 fflush(stdout);
-       }
-       else {
-	 rc = 1;
-	 fputs("Passwords must contain at least 8 characters.\n",stderr);
-	 fputs("1\n",stdout);
-	 fflush(stdout);
-       }
-    }
+  /* returns 0 if the password is long enough, otherwise non-zero  */
+main(int argc, char *argv[])
+{
+    char oldpassword[512];
+    char password[512];
+    int rc = 1;
+
+    if (fgets(oldpassword, 512, stdin))
+	while (fgets(password, 512, stdin)) {
+	    if (strlen(password) > 8) {	/* password includes a newline */
+		rc = 0;
+		fputs("0\n", stdout);
+		fflush(stdout);
+	    } else {
+		rc = 1;
+		fputs("Passwords must contain at least 8 characters.\n",
+		      stderr);
+		fputs("1\n", stdout);
+		fflush(stdout);
+	    }
+	}
+    return rc;
 }
