@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/update/client.c,v 1.11 2003/07/15 23:17:07 shadow Exp $");
+    ("$Header: /cvs/openafs/src/update/client.c,v 1.12.2.2 2007/10/30 15:24:08 shadow Exp $");
 
 #include <afs/stds.h>
 #ifdef	AFS_AIX32_ENV
@@ -36,13 +36,7 @@ RCSID
 #include <sys/time.h>
 #include <dirent.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -55,6 +49,9 @@ RCSID
 #include <afs/cellconfig.h>
 #include <afs/afsutil.h>
 #include <afs/fileutil.h>
+#ifdef	AFS_AIX_ENV
+#include <sys/statfs.h>
+#endif
 #include "update.h"
 #include "global.h"
 
@@ -69,8 +66,7 @@ static int RenameNewFiles(struct filestr *modFiles);
 static int PathsAreEquivalent(char *path1, char *path2);
 
 afs_int32
-GetServer(aname)
-     char *aname;
+GetServer(char *aname)
 {
     register struct hostent *th;
     afs_int32 addr;
@@ -98,9 +94,7 @@ osi_audit()
 #endif
 
 int
-main(argc, argv)
-     int argc;
-     char **argv;
+main(int argc, char **argv)
 {
     struct rx_connection *conn;
     struct rx_call *call;
@@ -218,7 +212,7 @@ main(argc, argv)
     errcode = rx_Init(0);
     if (errcode) {
 	printf("Rx initialize failed \n");
-	com_err(whoami, errcode, "calling Rx init");
+	afs_com_err(whoami, errcode, "calling Rx init");
 	exit(1);
     }
 
@@ -238,7 +232,7 @@ main(argc, argv)
 	exit(1);
     }
     if (errcode) {
-	com_err(whoami, errcode, "Couldn't get security obect for localAuth");
+	afs_com_err(whoami, errcode, "Couldn't get security obect for localAuth");
 	exit(1);
     }
 
@@ -260,7 +254,7 @@ main(argc, argv)
 
 	    /* construct local path from canonical (wire-format) path */
 	    if ((errcode = ConstructLocalPath(df->name, "/", &curDir))) {
-		com_err(whoami, errcode, "Unable to construct local path");
+		afs_com_err(whoami, errcode, "Unable to construct local path");
 		return errcode;
 	    }
 
@@ -271,7 +265,7 @@ main(argc, argv)
 #else
 		if (mkdir(curDir, 0700) < 0) {
 #endif
-		    com_err(whoami, errno, "can't create dir");
+		    afs_com_err(whoami, errno, "can't create dir");
 		    printf("upclient: Can't update dir %s\n", curDir);
 		    return -1;
 		}
@@ -290,14 +284,14 @@ main(argc, argv)
 	    error = rx_EndCall(call, 0);
 	    if (error && !errcode) {
 		printf("could not end rx call \n");
-		com_err(whoami, error, "calling EndCall");
+		afs_com_err(whoami, error, "calling EndCall");
 		goto fail;
 	    }
 	    if (errcode) {
 		printf
 		    ("warning: could not fetch the contents of directory %s \n",
 		     df->name);
-		com_err(whoami, errcode, "calling FetchFile");
+		afs_com_err(whoami, errcode, "calling FetchFile");
 		cnt++;
 		goto fail;
 	    }
@@ -305,7 +299,7 @@ main(argc, argv)
 	    stream = fopen(dirbuf, "r");
 	    if (stream == NULL) {
 		printf("fopen failed on %s \n", dirbuf);
-		com_err(whoami, errno, "fopen");
+		afs_com_err(whoami, errno, "fopen");
 		goto fail;
 	    }
 	    umask(00);
@@ -354,7 +348,7 @@ main(argc, argv)
 
 		dirp = opendir(curDir);
 		if (dirp == 0) {
-		    com_err(whoami, errno, "Can't open local dir %s", curDir);
+		    afs_com_err(whoami, errno, "Can't open local dir %s", curDir);
 		    goto fail;
 		}
 
@@ -373,7 +367,7 @@ main(argc, argv)
 			errcode = unlink(filename);
 			if (errcode) {
 			    printf("could not delete file %s \n", filename);
-			    com_err(whoami, errno, "could not delete file %s",
+			    afs_com_err(whoami, errno, "could not delete file %s",
 				    filename);
 			}
 		    }
@@ -405,10 +399,9 @@ main(argc, argv)
 }
 
 /* returns 1 if the file is upto date else returns 0*/
+/*check the dir case more carefully */
 int
-IsCompatible(filename, time, length)	/*check the dir case more carefully */
-     char *filename;
-     afs_int32 time, length;
+IsCompatible(char *filename, afs_int32 time, afs_int32 length)	
 {
     struct stat status;
     afs_int32 error;
@@ -416,7 +409,7 @@ IsCompatible(filename, time, length)	/*check the dir case more carefully */
 
     /* construct a local path from canonical (wire-format) path */
     if ((error = ConstructLocalPath(filename, "/", &localname))) {
-	com_err(whoami, error, "Unable to construct local path");
+	afs_com_err(whoami, error, "Unable to construct local path");
 	return error;
     }
 
@@ -425,7 +418,7 @@ IsCompatible(filename, time, length)	/*check the dir case more carefully */
     free(localname);
 
     if (error == -1)
-	return 0;		/*a non-existent file, has to be fetched fresh */
+	return 0;	/*a non-existent file, has to be fetched fresh */
     if ((status.st_mode & S_IFMT) == S_IFDIR
 	|| ((status.st_mtime == time) && (status.st_size == length)))
 	return (1);
@@ -434,10 +427,7 @@ IsCompatible(filename, time, length)	/*check the dir case more carefully */
 }
 
 int
-FetchFile(call, remoteFile, localFile, dirFlag)
-     struct rx_call *call;
-     char *localFile, *remoteFile;
-     int dirFlag;
+FetchFile(struct rx_call *call, char *remoteFile, char *localFile, int dirFlag)
 {
     int fd = -1, error = 0;
     struct stat status;
@@ -452,12 +442,12 @@ FetchFile(call, remoteFile, localFile, dirFlag)
     fd = open(localFile, O_CREAT | O_TRUNC | O_WRONLY, 0666);
     if (fd < 0) {
 	printf("Could not create %s\n", localFile);
-	com_err(whoami, errno, "Could not create %s", localFile);
+	afs_com_err(whoami, errno, "Could not create %s", localFile);
 	error = UPDATE_ERROR;
 	return error;
     }
     if (fstat(fd, &status) < 0) {
-	com_err(whoami, errno, "Could not stat %s", localFile);
+	afs_com_err(whoami, errno, "Could not stat %s", localFile);
 	close(fd);
 	printf("could not stast %s\n", localFile);
 	return UPDATE_ERROR;
@@ -473,33 +463,20 @@ FetchFile(call, remoteFile, localFile, dirFlag)
 
 
 int
-update_ReceiveFile(fd, call, status)
-     register int fd;
-     register struct rx_call *call;
-     register struct stat *status;
+update_ReceiveFile(register int fd, register struct rx_call *call, register struct stat *status)
 {
     register char *buffer = (char *)0;
     afs_int32 length;
-#ifdef notdef
-    XDR xdr;
-#endif
     register int blockSize;
     afs_int32 error = 0, len;
 #ifdef	AFS_AIX_ENV
-#include <sys/statfs.h>
     struct statfs tstatfs;
 #endif
 
-#ifdef	notdef
-    xdrrx_create(&xdr, call, XDR_DECODE);
-    if (!xdr_afs_int32(&xdr, &length))
-	return UPDATE_ERROR;
-#else
     len = rx_Read(call, &length, sizeof(afs_int32));
     length = ntohl(length);
     if (len != sizeof(afs_int32))
 	return UPDATE_ERROR;
-#endif
 #ifdef	AFS_AIX_ENV
     /* Unfortunately in AIX valuable fields such as st_blksize are gone from the stat structure!! */
     fstatfs(fd, &tstatfs);
@@ -520,7 +497,7 @@ update_ReceiveFile(fd, call, status)
 	if (!nbytes)
 	    error = UPDATE_ERROR;
 	if (write(fd, buffer, nbytes) != nbytes) {
-	    com_err(whoami, errno, "File system write failed!");
+	    afs_com_err(whoami, errno, "File system write failed!");
 	    printf("File system write failed!\n");
 	    error = UPDATE_ERROR;
 	}
@@ -585,9 +562,7 @@ PathsAreEquivalent(char *path1, char *path2)
  * deleted on client site) else it returns 0 */
 
 int
-NotOnHost(filename, okhostfiles)
-     char *filename;
-     struct filestr *okhostfiles;
+NotOnHost(char *filename, struct filestr *okhostfiles)
 {
     int i, rc;
     struct stat status;
@@ -605,7 +580,7 @@ NotOnHost(filename, okhostfiles)
     for (tf = okhostfiles; tf; tf = tf->next) {
 	/* construct local path from canonical (wire-format) path */
 	if ((rc = ConstructLocalPath(tf->name, "/", &hostfile))) {
-	    com_err(whoami, rc, "Unable to construct local path");
+	    afs_com_err(whoami, rc, "Unable to construct local path");
 	    return -1;
 	}
 	if (PathsAreEquivalent(hostfile, filename)) {
@@ -632,7 +607,7 @@ RenameNewFiles(struct filestr *modFiles)
     for (tf = modFiles; tf; tf = tf->next) {
 	/* construct local path from canonical (wire-format) path */
 	if ((errcode = ConstructLocalPath(tf->name, "/", &fname))) {
-	    com_err(whoami, errcode, "Unable to construct local path");
+	    afs_com_err(whoami, errcode, "Unable to construct local path");
 	    return errcode;
 	}
 	strcpy(newname, fname);
@@ -642,7 +617,7 @@ RenameNewFiles(struct filestr *modFiles)
 	errcode = renamefile(newname, fname);
 	if (errcode) {
 	    printf("could not rename %s to %s\n", newname, fname);
-	    com_err(whoami, errno, "could not rename %s to %s", newname,
+	    afs_com_err(whoami, errno, "could not rename %s to %s", newname,
 		    fname);
 	}
 	free(fname);
@@ -663,8 +638,7 @@ RenameNewFiles(struct filestr *modFiles)
  * and the uid, gid, file mode, access and modification times will be set to
  * the passed in values.
  */
-static
-    int
+static int
 GetFileFromUpServer(struct rx_connection *conn,	/* handle for upserver */
 		    char *filename,	/* name of file to be fetched */
 		    short uid, short gid,	/* uid/gid for fetched file */
@@ -684,7 +658,7 @@ GetFileFromUpServer(struct rx_connection *conn,	/* handle for upserver */
     /* construct local path from canonical (wire-format) path */
     errcode = ConstructLocalPath(filename, "/", &lfile);
     if (errcode) {
-	com_err(whoami, errcode, "Unable to construct local path");
+	afs_com_err(whoami, errcode, "Unable to construct local path");
 	return -1;
     }
     strcpy(newfile, lfile);
@@ -701,7 +675,7 @@ GetFileFromUpServer(struct rx_connection *conn,	/* handle for upserver */
 
     if (errcode) {
 	printf("failed to fetch file %s \n", filename);
-	com_err(whoami, errcode, "fetching file");
+	afs_com_err(whoami, errcode, "fetching file");
 	return 1;
     }
 
@@ -710,7 +684,7 @@ GetFileFromUpServer(struct rx_connection *conn,	/* handle for upserver */
     if (errcode) {
 	printf("could not change protection on %s to %u\n", newfile,
 	       (unsigned int)mode);
-	com_err(whoami, errno, "could not change protection on %s to %u",
+	afs_com_err(whoami, errno, "could not change protection on %s to %u",
 		newfile, mode);
 	return 1;
     }
@@ -723,7 +697,7 @@ GetFileFromUpServer(struct rx_connection *conn,	/* handle for upserver */
     if (errcode) {
 	printf("warning: could not change uid and gid on %s to %u and %u \n",
 	       newfile, gid, uid);
-	com_err(whoami, errno,
+	afs_com_err(whoami, errno,
 		"warning: could not change uid and gid on %s to %u and %u",
 		newfile, gid, uid);
     }
@@ -736,7 +710,7 @@ GetFileFromUpServer(struct rx_connection *conn,	/* handle for upserver */
     if (errcode) {
 	printf("could not change access and modify times on %s to %u %u\n",
 	       newfile, (unsigned int)atime, (unsigned int)mtime);
-	com_err(whoami, errno,
+	afs_com_err(whoami, errno,
 		"could not change access and modify times on %s to %u %u",
 		newfile, atime, mtime);
 	return 1;
